﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace PseudoCoverFlow
{
    public class AlbumCover : Control
    {
        public static DependencyProperty SourceProperty =
            DependencyProperty.Register("Source", typeof(ImageSource), typeof(AlbumCover), new PropertyMetadata(null, SourcePropertyChanged));

        private static void SourcePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AlbumCover self = d as AlbumCover;
            ImageSource value = e.NewValue as ImageSource;
            self.ApplySource(value);
        }

        public static DependencyProperty RotationYProperty =
            DependencyProperty.Register("RotationY", typeof(double), typeof(AlbumCover), new PropertyMetadata(0.0d, RotationYPropertyChanged));

        private static void RotationYPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AlbumCover self = d as AlbumCover;
            double value = (double)e.NewValue;
            self.ApplyRotationY(value);
        }

        public static DependencyProperty ScaleProperty =
            DependencyProperty.Register("Scale", typeof(double), typeof(AlbumCover), new PropertyMetadata(1.0d, ScalePropertyChanged));

        private static void ScalePropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AlbumCover self = d as AlbumCover;
            double value = (double)e.NewValue;
            self.ApplyScale(value);
        }

        public static DependencyProperty DarknessProperty =
            DependencyProperty.Register("Darkness", typeof(double), typeof(AlbumCover), new PropertyMetadata(0.0d, DarknessPropertyChanged));

        private static void DarknessPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AlbumCover self = d as AlbumCover;
            double value = (double)e.NewValue;
            self.ApplyDarkness(value);
        }

        public ImageSource Source
        {
            get
            {
                return (ImageSource)this.GetValue(SourceProperty);
            }
            set
            {
                this.SetValue(SourceProperty, value);
            }
        }

        public double RotationY
        {
            get
            {
                return (double)this.GetValue(RotationYProperty);
            }
            set
            {
                this.SetValue(RotationYProperty, value);
            }
        }

        public double Scale
        {
            get
            {
                return (double)this.GetValue(ScaleProperty);
            }
            set
            {
                this.SetValue(ScaleProperty, value);
            }
        }

        public double Darkness
        {
            get
            {
                return (double)this.GetValue(DarknessProperty);
            }
            set
            {
                this.SetValue(DarknessProperty, value);
            }
        }

        public AlbumCover()
        {
            DefaultStyleKey = typeof(AlbumCover);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            ApplySource(this.Source);
            ApplyRotationY(this.RotationY);
            ApplyScale(this.Scale);
            ApplyDarkness(this.Darkness);
        }

        private void ApplySource(ImageSource value)
        {
            Image image = this.GetTemplateChild("Image") as Image;
            Image reflection = this.GetTemplateChild("Reflection") as Image;
            if (image == null || reflection == null)
            {
                return;
            }
            image.Source = value;
            reflection.Source = value;
        }

        private void ApplyRotationY(double value)
        {
            PlaneProjection rotator = this.GetTemplateChild("Rotator") as PlaneProjection;
            if (rotator == null) return;
            rotator.RotationY = value;
        }

        private void ApplyScale(double value)
        {
            ScaleTransform scaler = this.GetTemplateChild("Scaler") as ScaleTransform;
            if (scaler == null) return;
            scaler.ScaleX = value;
            scaler.ScaleY = value;
        }

        private void ApplyDarkness(double value)
        {
            Border darkness = this.GetTemplateChild("Darkness") as Border;
            if (darkness == null) return;
            darkness.Opacity = value;
        }
    }
}
