﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Animation;

namespace PseudoCoverFlow
{
    public partial class MainPage : UserControl
    {
        private List<AlbumCover> covers = null;
        private List<List<Timeline>> lefts = null;
        private List<List<Timeline>> rights = null;
        private int selectedIndex = 0;
        private bool leftDown = false;
        private bool rightDown = false;
        private bool moving = false;

        public MainPage()
        {
            InitializeComponent();
            SetCovers();
            SetLefts();
            SetRights();
        }

        private void SetCovers()
        {
            covers = LayoutRoot.Children
                .Where(x => x is AlbumCover)
                .Select(x => (AlbumCover)x)
                .ToList();
        }

        private void SetLefts()
        {
            lefts = new List<List<Timeline>>();
            for (int i = 0; i < 10; i++)
            {
                string s = string.Format("P{0}", i);
                var l = Left.Children
                    .Where(x => Storyboard.GetTargetName(x) == s);
                lefts.Add(l.ToList());
            }
        }

        private void SetRights()
        {
            rights = new List<List<Timeline>>();
            for (int i = -1; i < 9; i++)
            {
                string s = string.Format("P{0}", i);
                var l = Right.Children
                    .Where(x => Storyboard.GetTargetName(x) == s);
                rights.Add(l.ToList());
            }
        }

        private void Move()
        {
            if (moving) return;
            if (!leftDown && !rightDown) return;
            if (leftDown && selectedIndex < covers.Count - 1)
            {
                moving = true;
                MoveLeft();
            }
            else if (rightDown && selectedIndex > 0)
            {
                moving = true;
                MoveRight();
            }
        }

        private void MoveLeft()
        {
            int centerOfLefts = (lefts.Count / 2) - 1;
            Left.Children.Clear();
            for (int i = 0; i < lefts.Count; i++)
            {
                int ci = selectedIndex - centerOfLefts + i;
                if (ci < 0 || covers.Count <= ci) continue;
                AlbumCover c = covers[ci];
                foreach (Timeline tl in lefts[i])
                {
                    Storyboard.SetTarget(tl, c);
                    Left.Children.Add(tl);
                }
            }
            Left.Begin();
        }

        private void MoveRight()
        {
            int centerOfRights = rights.Count / 2;
            Right.Children.Clear();
            for (int i = 0; i < rights.Count; i++)
            {
                int ci = selectedIndex - centerOfRights + i;
                if (ci < 0 || covers.Count <= ci) continue;
                AlbumCover c = covers[ci];
                foreach (Timeline tl in rights[i])
                {
                    Storyboard.SetTarget(tl, c);
                    Right.Children.Add(tl);
                }
            }
            Right.Begin();
        }

        private void LeftButton_MouseEnter(object sender, MouseEventArgs e)
        {
            leftDown = true;
            Move();
        }

        private void LeftButton_MouseLeave(object sender, MouseEventArgs e)
        {
            leftDown = false;
            Move();
        }

        private void RightButton_MouseEnter(object sender, MouseEventArgs e)
        {
            rightDown = true;
            Move();
        }

        private void RightButton_MouseLeave(object sender, MouseEventArgs e)
        {
            rightDown = false;
            Move();
        }

        private void Left_Completed(object sender, EventArgs e)
        {
            Left.Stop();
            ++selectedIndex;
            SetPosition();
            moving = false;
            Move();
        }

        private void Right_Completed(object sender, EventArgs e)
        {
            Right.Stop();
            --selectedIndex;
            SetPosition();
            moving = false;
            Move();
        }

        private void SetPosition()
        {
            for (int i = 0; i < covers.Count; i++)
            {
                AlbumCover c = covers[i];
                int pi = i - selectedIndex + 4;
                if (pi < 0)
                {
                    pi = -1;
                }
                else if (9 <= pi)
                {
                    pi = 9;
                }
                SetPosition(c, pi);
            }
        }

        private void SetPosition(AlbumCover c, int pi)
        {
            c.SetValue(Canvas.LeftProperty, (double)(pi * 50 + ((pi < 4) ? 60 : (pi == 4) ? 160 : 260)));
            c.SetValue(Canvas.ZIndexProperty, (pi == 4) ? 1 : -Math.Abs(pi - 4));
            c.Scale = (pi == 4) ? 0.45 : 0.32;
            c.RotationY = 53.13 * Math.Sign(pi - 4);
            c.Darkness = (pi == 4) ? 0.0 : 0.5;
        }
    }
}
